@@ -34,25 +34,29 @@ def resolve_blocks(template, context, blocks=None):
 
     return blocks
 
-class TempContext(object):
-    '''A context manager to make it easy to push context temporarily'''
-    def __init__(self, context, update):
+
+class ContextDict(dict):
+    '''Back-port of the ContextDict from Django 1.7'''
+    def __init__(self, context, *args, **kwargs):
+        super(ContextDict, self).__init__(*args, **kwargs)
+
+        context.dicts.append(self)
         self.context = context
-        self.update = update
 
     def __enter__(self):
-        self.context.update(self.update)
+        self.context.update(self)
         return self.context
 
-    def __exit__(self, exc_type, exc_value, traceback):
+    def __exit__(self, *args, **kwargs):
         self.context.pop()
 
+
 @register.tag
 def form(parser, token):
     '''Prepare to render a Form, using the specified template.
 
     {% form "template.form" %}
-        {% field form.somefield "blockname" ..... %}
+        {% field "blockname" form.somefield ..... %}
         ...
     {% endform %}
     '''
@@ -89,22 +93,57 @@ class FormNode(template.Node):
         kwargs['formulation'] = resolve_blocks(tmpl_name, context)
 
         # Render our children
-        with TempContext(context, kwargs) as context:
+        with ContextDict(context, kwargs) as context:
             return self.nodelist.render(context)
 
+def _auto_widget(field, context):
+    # Auto-detect
+    info = {
+        'widget': field.field.widget.__class__.__name__,
+        'field': field.field.__class__.__name__,
+        'name': field.name,
+    }
+
+    for pattern in (
+            '{field}_{widget}_{name}',
+            '{field}_{name}',
+            '{widget}_{name}',
+            '{field}_{widget}',
+            '{name}',
+            '{widget}',
+            '{field}',
+        ):
+        block = context['formulation'].get_block(
+            pattern.format(info)
+        )
+        if block is not None:
+            return block
+    raise TemplateSyntaxError("Could not find widget for field: %r" % field)
+    
 
 @register.simple_tag(takes_context=True)
-def field(context, widget, *fields, **kwargs):
-    kwargs['field'] = fields[0]
-    kwargs['fields'] = fields
+def field(context, field, widget=None, **kwargs):
+    field_data = {
+        'form_field': field,
+        'id': field.auto_id,
+    }
+    for attr in ('css_classes', 'errors', 'field', 'form',
+            'help_text', 'id_for_label', 'label', 'name', 'html_name',
+            'value',):
+        field_data[attr] = getattr(field, attr)
+    for attr in ('choices', 'widget', 'required'):
+        field_data[attr] = getattr(field.field, attr, None)
+    kwargs.update(field_data)
+    if widget is None:
+        widget = _auto_widget(field, context)
     kwargs['block'] = block = context['formulation'].get_block(widget)
-    with TempContext(context, kwargs) as context:
+    with ContextDict(context, kwargs) as context:
         return block.render(context)
 
 
 @register.simple_tag(takes_context=True)
 def use(context, widget, **kwargs):
     kwargs['block'] = block = context['formulation'].get_block(widget)
-    with TempContext(context, kwargs) as context:
+    with ContextDict(context, kwargs) as context:
         return block.render(context)
 
